# Internal class for Puppet Enterprise to configure the Puppet Server
#
# @param metrics_server_id [Integer] What ID to tag the metrics with
# @param metrics_jmx_enabled [Boolean] Toggle whether to run with JMX (Java Management Extensions) enabled.
# @param metrics_graphite_enabled [Boolean] Toggle whether to write metrics to Graphite.
# @param metrics_graphite_host [String] Graphite server to write metrics to..
# @param metrics_graphite_port [Integer] Port on Graphite server to write metrics to.
# @param metrics_graphite_update_interval_seconds [Integer] How often, in seconds, to wait between writing metrics to the Graphity server.
# @param profiler_enabled [Boolean] Whether to run the Java profiler.
# @param jruby_max_active_instances [Integer] The maximum number of jruby instances that the Puppet Server will spawn to server Agent traffic.
# @param jruby_max_requests_per_instance [Integer] The maximum number of requests a single JRuby instance will handle before it is flushed from memory and refreshed. 
# @param jruby_borrow_timeout [Integer] Set the timeout when attempting to borrow an instance from the JRuby pool in milliseconds. Defaults is 1200000.
# @param certname [String] The cert name the Server will use.
# @param localcacert [String] The path to the local CA cert used for generating SSL certs.
# @param puppet_admin_certs [Array] Certificates to add to the whitelist on the master.
# @param static_files [Hash] Paths to static files to serve up on the master. Keys are the URLs the files will be found at, values are the paths to the folders to be served.
# @param java_args [Hash] Key-value pairs describing the java arguments to be passed when starting the master
# @param tk_jetty_max_threads [Integer] Maxiumum number of Jerry threads that should be started
# @param ssl_protocols [Array] List of acceptable protocols for making HTTP requests
# @param cipher_suites [Array] A list of acceptable cipher suites for making HTTP requests
# @param idle_timeout_milliseconds [Integer] The amount of time an outbound HTTP connection will wait for data to be available before closing the socket
# @param connect_timeout_milliseconds [Integer] The amount of time an outbound HTTP connection will wait to connect before giving up
class puppet_enterprise::master::puppetserver(
  $metrics_server_id,
  $metrics_jmx_enabled,
  $metrics_graphite_enabled,
  $metrics_graphite_host,
  $metrics_graphite_port,
  $metrics_graphite_update_interval_seconds,
  $profiler_enabled,
  $ssl_protocols                             = undef,
  $cipher_suites                             = undef,
  $idle_timeout_milliseconds                 = undef,
  $connect_timeout_milliseconds              = undef,
  $jruby_max_active_instances                = undef,
  $jruby_max_requests_per_instance           = undef,
  $jruby_borrow_timeout                      = undef,
  $certname                                  = $::clientcert,
  $localcacert                               = $puppet_enterprise::params::localcacert,
  $puppet_admin_certs                        = [],
  $static_files                              = {},
  $java_args                                 = $puppet_enterprise::params::puppetserver_java_args,
  $tk_jetty_max_threads                      = undef,
) inherits puppet_enterprise::params {

  pe_validate_array($puppet_admin_certs)

  if $ssl_protocols {
    pe_validate_array($ssl_protocols)
  }

  if $cipher_suites {
    pe_validate_array($cipher_suites)
  }

  if $idle_timeout_milliseconds {
    pe_validate_single_integer($idle_timeout_milliseconds)
  }

  if $connect_timeout_milliseconds {
    pe_validate_single_integer($connect_timeout_milliseconds)
  }

  if $jruby_max_requests_per_instance {
    pe_validate_single_integer($jruby_max_requests_per_instance)
  }

  $classifier_client_certname = $puppet_enterprise::params::classifier_client_certname
  $container = 'puppetserver'
  $_puppet_admin_certs = pe_union([$classifier_client_certname], $puppet_admin_certs)

  File {
    ensure => present,
    owner  => 'pe-puppet',
    group  => 'pe-puppet',
    mode   => '0640',
  }

  Pe_concat {
    owner  => 'pe-puppet',
    group  => 'pe-puppet',
    mode   => '0640',
  }

  Puppet_enterprise::Trapperkeeper::Bootstrap_cfg {
    container => $container,
  }

  $confdir = "/etc/puppetlabs/${container}"

  puppet_enterprise::trapperkeeper::bootstrap_cfg { "${container}:master jetty9-service" :
    namespace => 'puppetlabs.trapperkeeper.services.webserver.jetty9-service',
    service   => 'jetty9-service',
  }

  puppet_enterprise::trapperkeeper::bootstrap_cfg { 'pe-master-service' :
    namespace => 'puppetlabs.enterprise.services.master.master-service',
  }

  puppet_enterprise::trapperkeeper::bootstrap_cfg { 'request-handler-service' :
    namespace => 'puppetlabs.services.request-handler.request-handler-service',
  }
  puppet_enterprise::trapperkeeper::bootstrap_cfg { 'jruby-puppet-pooled-service' :
    namespace => 'puppetlabs.services.jruby.jruby-puppet-service',
  }

  puppet_enterprise::trapperkeeper::bootstrap_cfg { 'metrics-puppet-profiler-service' :
    namespace => 'puppetlabs.enterprise.services.puppet-profiler.puppet-profiler-service',
  }

  puppet_enterprise::trapperkeeper::bootstrap_cfg { 'pe-metrics-service' :
    namespace => 'puppetlabs.enterprise.services.metrics.pe-metrics-service',
  }

  puppet_enterprise::trapperkeeper::bootstrap_cfg { 'puppet-server-config-service' :
    namespace => 'puppetlabs.services.config.puppet-server-config-service',
  }

  puppet_enterprise::trapperkeeper::bootstrap_cfg { 'puppet-admin-service' :
    namespace => 'puppetlabs.services.puppet-admin.puppet-admin-service',
  }

  puppet_enterprise::trapperkeeper::bootstrap_cfg { 'webrouting-service' :
    namespace => 'puppetlabs.trapperkeeper.services.webrouting.webrouting-service',
  }

  puppet_enterprise::trapperkeeper::bootstrap_cfg { 'scheduler-service' :
    namespace => 'puppetlabs.trapperkeeper.services.scheduler.scheduler-service',
  }

  puppet_enterprise::trapperkeeper::bootstrap_cfg { 'status-service' :
    namespace => 'puppetlabs.trapperkeeper.services.status.status-service',
  }

  puppet_enterprise::trapperkeeper::bootstrap_cfg { 'pe-jruby-metrics-service' :
    namespace => 'puppetlabs.enterprise.services.jruby.pe-jruby-metrics-service',
  }

  # Uses
  #   $certname
  #   $localcacert
  #   $static_files
  #   $tk_jetty_max_threads
  file { "${confdir}/conf.d/webserver.conf" :
    content => template('puppet_enterprise/master/puppetserver/webserver.conf.erb'),
    notify => Service["pe-${container}"],
  }

  file { "${confdir}/conf.d/web-routes.conf" :
    content => template('puppet_enterprise/master/puppetserver/web-routes.conf.erb'),
    notify  => Service["pe-${container}"],
  }

  # Uses
  #   $profiler_enabled
  #   $_puppet_admin_certs
  #   $ssl_protocols
  #   $cipher_suites
  #   $idle_timeout_milliseconds
  #   $connect_timeout_milliseconds
  #   $jruby_max_requests_per_instance
  file { "${confdir}/conf.d/pe-puppet-server.conf" :
    content => template('puppet_enterprise/master/puppetserver/puppet-server.conf.erb'),
    notify => Service["pe-${container}"],
  }

  # Uses
  #   $confdir
  #   $certname
  file { "${confdir}/conf.d/global.conf" :
    content => template('puppet_enterprise/master/puppetserver/global.conf.erb'),
    notify => Service["pe-${container}"],
  }

  # Uses
  #   $metrics_server_id
  #   $metrics_jmx_enabled
  #   $metrics_graphite_enabled
  #   $metrics_graphite_host
  #   $metrics_graphite_port
  #   $metrics_graphite_update_interval_seconds
  file { "${confdir}/conf.d/metrics.conf" :
    content => template('puppet_enterprise/master/puppetserver/metrics.conf.erb'),
    notify => Service["pe-${container}"],
  }

  if !pe_empty($java_args) {
    pe_validate_hash($java_args)
    $puppetserver_initconf = "${puppet_enterprise::params::defaults_dir}/pe-puppetserver"
    create_resources(
      'pe_ini_subsetting',
      create_java_args_subsettings_hash(
        'pe-puppetserver',
        $java_args,
        { path => $puppetserver_initconf,
          require => Package['pe-puppetserver'],
          notify => Service["pe-${container}"],
        })
    )
  }

  service { "pe-${container}":
    ensure    => running,
    enable    => true,
  }
}
